<?php

/** PHP command-line script to generate a webpage with previews of a font
 *  collection.
 *
 * This script will generate an image for each font with the given preview
 * text. A simple HTML document will be generated that displays each preview
 * alongside the font filename.
 *
 * This tool will act recursively from the current directory, and searches
 * only for *.ttf files.
 *
 * Usage:
 *
 * $ cd /my/fonts
 * $ php ~/bin/fontpreview.php
 * $ x-www-browser index.html
 *
 * Open issues:
 * - Does not detect hash collisions for subdirectories. Highly unlikely,
 *   and also requires font name to collide.
 *
 * Requirements:
 * - GD library for PHP is required.
 *
 *
 * (c) 2010 Ben Stewart. All rights reserved.
 *
 * This is released under the terms of the BSD license. See COPYRIGHT for
 * further information.
 */

define('PREVIEW_PATH', '.font_preview');
define('HTML_FILENAME', 'index.html');
define('PREVIEW_TEXT', 'The quick brown fox jumps over the lazy dog');
define('PREVIEW_SIZE', 24);

putenv('GDFONTPATH=' . realpath('.'));

/** Perform a depth-first traversal of the directory tree and preview fonts.
 *
 * @param  dName        Directory name to start from
 * @param  sampleText   Text to display in the preview image.
 * @param  htmlFile     Open file descriptor to write the HTML to.
 */
function previewDir($dName, $sampleText, $htmlFile)
{
    if ($d = opendir($dName))
    {
        while ($f = readdir($d))
        {
            /* If we think this might be a font.. */
            if (is_file($dName . '/' . $f) &&
                preg_match('/^(.*)\.ttf$/i', $f, $regs))
            {
                /* Test that we can use the font, and at the same time, find
                 * the bounding box for the text string.
                 */
                if (is_array($bbox = @imagettfbbox(PREVIEW_SIZE, 0, $dName . '/' . $f, $sampleText)))
                {
                    /* Create a GD image. */
                    $canvas = imagecreatetruecolor($bbox[2] - $bbox[0], $bbox[3] - $bbox[5]);
                    $black = imagecolorallocate($canvas, 0, 0, 0);
                    $white = imagecolorallocate($canvas, 255, 255, 255);

                    /* Fill the background, display the text on top of it. */
                    imagefilledrectangle($canvas, 0, 0, $bbox[2] - $bbox[0], $bbox[3] - $bbox[5], $white);
                    imagettftext($canvas, PREVIEW_SIZE, 0, 0, 0 - $bbox[5], $black, $dName . '/' . $f, $sampleText);

                    /* Save the preview image to a .png in the preview dir, clean up */
                    imagepng($canvas, PREVIEW_PATH . '/' . md5($dName) . '_' . $f . '.png');
                    imagedestroy($canvas);

                    /* Refer to the preview in the HTML file. */
                    fputs($htmlFile, '<h1>' . htmlspecialchars($dName . '/' . $f) . "</h1>\n");
                    fputs($htmlFile, '<img src="' . htmlspecialchars(str_replace('+','%20', urlencode(PREVIEW_PATH) . '/' . urlencode(md5($dName) . '_' . $f))) . '.png">');
                }
                else
                {
                    /* Display a warning if we can't use the font. */
                    fputs(STDERR, "Font " . $dName . "/" . $f . " is damaged.\n");
                }
            }
            elseif (is_dir($dName . '/' . $f) && !preg_match('/^\./', $f))
            {
                /* Recurse into a subdirectory */
                previewDir($dName . '/' . $f, $sampleText, $htmlFile);
            }
        }

        closedir($d);
    }
}


function generateFontPreviews($directory, $sampleText, $htmlFilename)
{
    if (!function_exists('imagettfbbox'))
        die("The GD module is not installed.\n");

    /* Create font preview image directory, check that it was created */
    if (!is_dir(PREVIEW_PATH))
        @mkdir(PREVIEW_PATH);

    if (!is_dir(PREVIEW_PATH))
      die("Could not find preview directory\n");

    /* Try to open the HTML file, and dump the header. */
    if (!($htmlFile = fopen(HTML_FILENAME, 'w')))
      die("Could not open preview HTML.\n");

    fputs($htmlFile,
        "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\">\n" .
        "<html>\n" .
        "    <head>\n" . 
        "        <title>Font Preview</title>" .
        "        <style type=\"text/css\"><!--\n" .
        "            h1 { color: #779; margin-top: 1.5em; }\n" . 
        "        --></style>\n" .
        "    </head>\n" .
        "    <body>\n");

    /* Generate the previews */
    previewDir($directory, $sampleText, $htmlFile);

    /* Dump out the footer */
    fputs($htmlFile,
        "    </body>\n" .
        "</html>\n");

    fclose($htmlFile);
}

generateFontPreviews('.', PREVIEW_TEXT, HTML_FILENAME);

?>
